from zope.interface import directlyProvides
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleVocabulary
from collective.teaser import MsgFact as _
from plone.app.imaging.utils import getAllowedSizes


def ImportanceVocabulary(context):
    items =[
        (_(u"importance_highest", default=u"Highest importance"),'4'),
        (_(u"importance_high", default=u"High importance"),'3'),
        (_(u"importance_normal", default=u"Normal importance"),'2'),
        (_(u"importance_low", default=u"Low importance"),'1')]
    return SimpleVocabulary.fromItems(items)

directlyProvides(ImportanceVocabulary, IVocabularyFactory)


def ImageScaleVocabulary(context):
    allowed_sizes = getAllowedSizes()
    items = [(u'%s(%s, %s)' %(key, value[0], value[1]), key)
        for key,value in allowed_sizes.items() if allowed_sizes]
    return SimpleVocabulary.fromItems(items)

directlyProvides(ImageScaleVocabulary, IVocabularyFactory)



from zope.i18nmessageid import MessageFactory
from Products.CMFCore import utils
try:
    from Products.LinguaPlone import public  as atapi
except ImportError:
    # No multilingual support
    from Products.Archetypes import atapi
from collective.teaser import config


MsgFact = MessageFactory(config.PROJECTNAME)


def initialize(context):
    """Register content types through Archetypes with Zope and the CMF.
    """
    from collective.teaser.content import teaser

    content_types, constructors, ftis = atapi.process_types(
        atapi.listTypes(config.PROJECTNAME),
        config.PROJECTNAME)

    for atype, constructor in zip(content_types, constructors):
        utils.ContentInit("%s: %s" % (config.PROJECTNAME, atype.portal_type),
            content_types      = (atype,),
            permission         = config.ADD_PERMISSIONS[atype.portal_type],
            extra_constructors = (constructor,),
            ).initialize(context)



from zope.interface import Interface
from plone.portlets.interfaces import IPortletManager


class ITeaser(Interface):
    """Marker interface.
    """

class ITeaserLayer(Interface):
    """Browser Layer for teaser.
    """

class ITeaserPortletManager(IPortletManager):
    """Teaser Portlet Manager
    """



from zope.interface import implements
from AccessControl import ClassSecurityInfo
from Products.CMFCore.permissions import View
try:
    from Products.LinguaPlone import public  as atapi
except ImportError:
    # No multilingual support
    from Products.Archetypes import atapi
from archetypes.referencebrowserwidget.widget import ReferenceBrowserWidget
from Products.ATContentTypes.content import base, image, schemata
from Products.ATContentTypes.lib.historyaware import HistoryAwareMixin
from collective.teaser.interfaces import ITeaser
from collective.teaser.config import PROJECTNAME, DEFAULT_IMPORTANCE
from collective.teaser import MsgFact as _
from plone.app.imaging.utils import getAllowedSizes


allowed_sizes = getAllowedSizes()


type_schema = schemata.ATContentTypeSchema.copy() + atapi.Schema((

    atapi.ImageField('image',
        required=False,
        languageIndependent=True,
        sizes=allowed_sizes,
        widget=atapi.ImageWidget(
            label=_(u"label_image", default=u"Image"),
            description=_(u"help_image",
                          default=u"Image to display as teaser."),
        ),
    ),

    atapi.ReferenceField('link_internal',
        required=False,
        searchable=False,
        languageIndependent=True,
        multiValued=False,
        relationship='ref_link_internal',
        widget=ReferenceBrowserWidget(
            label=_(u"label_link_internal", default=u"Internal Link"),
            description=_(u"help_link_internal",
                          default=u"Link to internal content. For external " +\
                                  u"content, use the External Link field."),
            allow_search=True,
            allow_browse=True,
            force_close_on_insert = True,
            startup_directory = '/',
        ),
    ),

    atapi.StringField('link_external',
        required=False,
        searchable=False,
        languageIndependent=True,
        validators=("isURL"),
        widget=atapi.StringWidget(
            label=_(u"label_link_external", default=u"External Link"),
            description=_(u"help_link_external",
                          default=u"Url to external content. For internal " +\
                                  u"content, use the field above. Use the " +\
                                  u"form http://WEBSITE.TLD/"),
        ),
    ),

    atapi.StringField('importance',
        required=True,
        searchable=True,
        languageIndependent=True,
        vocabulary_factory="collective.teaser.ImportanceVocabulary",
        enforceVocabulary=True,
        default=DEFAULT_IMPORTANCE,
        widget=atapi.SelectionWidget(
            label=_(u"label_importance", default=u"Importance"),
            description=_(u"help_importance",
                          default=u"Select the importance of the teaser. " +\
                                  u"The frequency of the teaser will be " +\
                                  u"set accordingly."),
        ),
    ),
))

schemata.finalizeATCTSchema(type_schema,
                            folderish=False,
                            moveDiscussion=False)


class Teaser(base.ATCTContent, image.ATCTImageTransform, HistoryAwareMixin):
    implements(ITeaser)
    portal_type = "Teaser"
    _at_rename_after_creation = True
    schema = type_schema
    security = ClassSecurityInfo()

    security.declareProtected(View, 'tag')
    def tag(self, **kwargs):
        """Generate image tag using the api of the ImageField
        """
        if 'title' not in kwargs:
            kwargs['title'] = self.title
        return self.getField('image').tag(self, **kwargs)

atapi.registerType(Teaser, PROJECTNAME)



PROJECTNAME = "collective.teaser"
DEFAULT_IMPORTANCE = '3',
ADD_PERMISSIONS = {
    "Teaser"        : "collective.teaser: Add Teaser",
}

from Products.CMFCore.permissions import setDefaultRoles
DEFAULT_ADD_CONTENT_PERMISSION = "Add portal content"
setDefaultRoles(DEFAULT_ADD_CONTENT_PERMISSION,
                ('Manager', 'Owner', 'Contributor'))



# -*- coding: utf-8 -*-
import uuid
from node.utils import instance_property
from zope.component import (
    getUtility,
    getMultiAdapter
)
from zope.formlib import form
from zope.interface import implements
from zope import schema
from zope.pagetemplate.pagetemplatefile import PageTemplateFile
from plone.memoize import ram
from plone.portlets.interfaces import IPortletDataProvider
from plone.app.portlets.cache import get_language
from plone.app.portlets.portlets import base
from plone.app.portlets.interfaces import (
    IPortletManager,
    IPortletAssignmentMapping,
)
from Acquisition import (
    aq_inner,
    aq_parent,
)
from Products.Five import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.interfaces import IPloneSiteRoot
from collective.teaser.config import DEFAULT_IMPORTANCE
from collective.teaser import MsgFact as _
from collective.teaser.browser.common import get_teasers



def render_cachekey(fun, self):
    """
    Based on render_cachekey from plone.app.portlets.cache, without the
    fingerprint based on the portlet's catalog brains.

    Generates a key based on:

    * Portal URL
    * Negotiated language
    * Anonymous user flag
    * Portlet manager
    * Assignment

    """
    context = aq_inner(self.context)
    anonymous = getToolByName(context, 'portal_membership').isAnonymousUser()

    return "".join((
        getToolByName(aq_inner(self.context), 'portal_url')(),
        get_language(aq_inner(self.context), self.request),
        str(anonymous),
        self.manager.__name__,
        self.data.__name__))

def get_portlet_assingment(context, uid):
    for name in [u"plone.leftcolumn", u"plone.rightcolumn",
                 u"collective.teaser.portletmanager"]:
        manager = getUtility(IPortletManager, name=name)
        for category in manager.values():
            for group in category.values():
                for assignment in group.values():
                    if ITeaserPortlet.providedBy(assignment):
                        if uid == str(assignment.uid):
                            return assignment
        context = aq_inner(context)
        while True:
            try:
                assignment_mapping = getMultiAdapter((context, manager),
                                                     IPortletAssignmentMapping)
            except:
                return
            for assignment in assignment_mapping.values():
                if ITeaserPortlet.providedBy(assignment):
                    if uid == str(assignment.uid):
                        return assignment
            if IPloneSiteRoot.providedBy(context):
                break
            context = aq_parent(aq_inner(context))


class TeaserRenderer(object):
    template = PageTemplateFile('teaser.pt')

    def __init__(self, context, data, request):
        self.context = context
        self.data = data
        self.request = request

    def __call__(self):
        return self.template(options=self)

    @instance_property
    def teasers(self):
        return get_teasers(self.context, self.data, self.request)


class AjaxTeaser(BrowserView):

    def __call__(self):
        return TeaserRenderer(self.context, self.data, self.request)()

    @property
    def data(self):
        return get_portlet_assingment(self.context, self.request.get('uid'))


class ITeaserPortlet(IPortletDataProvider):

    importance_levels = schema.Tuple(
        title=_(u'portlet_label_importance_levels',
                default=u'Importance Levels'),
        description=_(u'portlet_help_importance_levels',
                      default=u'Select which importance levels the portlet '
                              u'should show.'),
        default=(DEFAULT_IMPORTANCE,),
        required=True,
        value_type=schema.Choice(
            vocabulary="collective.teaser.ImportanceVocabulary"),
        )

    keywords_filter = schema.Tuple(
        title=_(u'portlet_label_keywords_filter',
                default=u'Keywords Filter'),
        description=_(u'portlet_help_keywords_filter',
                      default=u'Select which teasers with specific keywords '
                              u'should be shown. Select none to order to show '
                              u'any teasers.'),
        default=None,
        required=False,
        value_type=schema.Choice(
            vocabulary="plone.app.vocabularies.Keywords"),
        )

    teaser_scale = schema.Choice(
        title=_(u'portlet_label_image_scale', default=u'Image Scale'),
        description=_(u'portlet_help_image_scale',
                      default=u'Select, which image scale should be used '
                              u'for the portlet.'),
        required=True,
        default=None,
        vocabulary="collective.teaser.ImageScaleVocabulary",
        )

    num_teasers = schema.Int(
        title=_(u'portlet_label_num_teasers', default=u'Number of teasers'),
        description=_(u'portlet_help_num_teasers',
                      default=u'Define the maximum number of teasers, which '
                              u'are displayed in this portlet'),
        default=1,
        )

    ajaxified = schema.Bool(
        title=_(u'portlet_label_ajaxified', default=u'Load Teasers via AJAX?'),
        description=_(u'portlet_help_ajaxified',
                      default=u'Whether teaser is loaded deferred via ajax.'
                              u'or directly.'),
        default=True,
        )


    show_title = schema.Bool(
        title=_(u'portlet_label_show_title', default=u'Show title'),
        description=_(u'portlet_help_show_title',
                      default=u'Show the title of the teaser.'),
        default=True,
        )

    show_description = schema.Bool(
        title=_(u'portlet_label_show_description', default=u'Show description'),
        description=_(u'portlet_help_show_description',
                      default=u'Show the description of the teaser as text ' +\
                              u'below the image.'),
        default=False,
        )


class Renderer(base.Renderer):
    render = ViewPageTemplateFile('teaser_portlet.pt')

    # On default pages, portlets are called twice (parent and default page).
    # When the teaser is going to be displayed on a default page, the teaser is
    # called twice. This seems to be a bug/default-page-sideeffect in Plone.
    # When only one teaser is available for that portlet, on the second call
    # it's removed from the available teasers, since it's already in the
    # taken_teasers list. As a result, no teaser is shown. Therefore, we cache
    # the render call.
    @ram.cache(render_cachekey) # cached per request
    def renderer(self):
        return TeaserRenderer(self.context, self.data, self.request)

    @property
    def display(self):
        return bool(self.renderer().teasers)

    @property
    def rendered_teasers(self):
        return self.renderer()


class Assignment(base.Assignment):
    implements(ITeaserPortlet)

    # avoid upgrade pain
    show_description = False
    keywords_filter = None

    def __init__(self, importance_levels=None,
            keywords_filter=None,
            teaser_scale=None,
            num_teasers=1,
            ajaxified=True,
            show_title=True,
            show_description=False):
        self.importance_levels = importance_levels
        self.keywords_filter = keywords_filter
        self.teaser_scale = teaser_scale
        self.num_teasers=num_teasers
        self.ajaxified = ajaxified
        self.show_title=show_title
        self.show_description=show_description

        self.uid = uuid.uuid4()

    @property
    def title(self):
        return _(u'portlet_teaser_title', default=u"Teaser")


class AddForm(base.AddForm):
    form_fields = form.Fields(ITeaserPortlet)
    label = _(u'portlet_label_add', default=u"Add portlet to show teasers.")
    description = _(u'portlet_help_add', default=u"This portlet shows teasers.")

    def create(self, data):
        return Assignment(**data)


class EditForm(base.EditForm):
    form_fields = form.Fields(ITeaserPortlet)
    label = _(u'portlet_label_add', default=u"Add portlet to show teasers.")
    description = _(u'portlet_help_add', default=u"This portlet shows teasers.")



# XXX: remove

#from plone.memoize.ram import global_cache
#
#def cache_invalidate_teaserlist(obj, event):
#    """ Invalidate the _teaserlist cache after updating or adding a teaser.
#    """
#    global_cache.invalidate(
#            'collective.teaser.browser.common._teaserlist')



# -*- coding: utf-8 -*-
from zope.component import adapts
from zope.interface import Interface
from zope.publisher.interfaces.browser import IBrowserView
from zope.publisher.interfaces.browser import IDefaultBrowserLayer
from plone.app.portlets.manager import ColumnPortletManagerRenderer
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from collective.teaser.interfaces import ITeaserPortletManager


class TeaserPortletManagerRenderer(ColumnPortletManagerRenderer):
    adapts(Interface, IDefaultBrowserLayer, IBrowserView, ITeaserPortletManager)
    template = ViewPageTemplateFile('teaser_portlet_manager_renderer.pt')


# -*- coding: utf-8 -*-
import random
from Acquisition import aq_inner
from DateTime import DateTime
from Products.CMFCore.utils import getToolByName


def _teaserlist(context, data):
    """XXX: cache on request.
    """
    context = aq_inner(context)
    cat = getToolByName(context,'portal_catalog')
    query = {}
    query['Type'] = 'Teaser'

    # show only selected importances
    query['importance'] = data.importance_levels

    if data.keywords_filter:
        query['Subject'] = data.keywords_filter

    # show only published and not expired, even for admins
    query['review_state'] = 'published'
    query['effectiveRange'] = DateTime()
    brains = cat(**query)

    # make a weighted (multiplied by importance) list of teasers.
    teasers = []
    [teasers.extend(int(teaser.importance) * [teaser]) for teaser in brains]
    return teasers


def get_teasers(context, data, request):
    teasers = _teaserlist(context, data)
    if not teasers: return None

    # get used id's from request and exclude them
    taken_teasers = getattr(request, 'teasers', [])

    choosen_teasers = []
    for cnt in range(data.num_teasers):
        # reduce selected teasers with all taken_teasers
        teasers = [_ for _ in teasers if _.id not in taken_teasers]
        if not teasers:
            break

        # randomly select num_teasers from all
        choosen_teaser = random.choice(teasers)
        choosen_teasers.append(choosen_teaser.getObject())
        taken_teasers.append(choosen_teaser.id)

    # save new taken teaser list in request
    request['teasers'] = taken_teasers

    # create data structure and return
    show_title = data.show_title
    show_desc = data.show_description
    scale = data.teaser_scale
    teaser_list = []
    for teaser in choosen_teasers:
        img_text_part = not show_desc and teaser.Description() or ''
        img_text = '%s%s' % (teaser.title,
                         img_text_part and ' - %s' % img_text_part or '')
        teaser_list.append({
            'title': show_title and teaser.title or None,
            'image': getattr(teaser, 'image', False) \
                         and teaser.getField('image').tag(
                             teaser,
                             scale=scale,
                             alt=img_text,
                             title=img_text) or None,
             'description': show_desc and teaser.Description() or None,
             'url': teaser.getLink_internal() \
                        and teaser.getLink_internal().absolute_url() \
                        or teaser.link_external or None,
        })
    return teaser_list



# -*- coding: utf-8 -*-
from zope.component import getMultiAdapter
from plone.app.layout.viewlets.common import ViewletBase
from Products.CMFCore.utils import getToolByName


class TeaserPortletsViewlet(ViewletBase):

    name = 'Teaser portlets'
    manage_view = '@@manage-teaserportlets'

    @property
    def display(self):
        return True

    def update(self):
        if not self.display:
            self.canManagePortlets = False
            return
        context_state = getMultiAdapter(
            (self.context, self.request), name=u'plone_context_state')
        self.manageUrl = '%s/%s' % (context_state.view_url(), self.manage_view)
        # This is the way it's done in plone.app.portlets.manager, so we'll
        # do the same
        mt = getToolByName(self.context, 'portal_membership')
        self.canManagePortlets = mt.checkPermission(
            'Portlets: Manage portlets', self.context)



from setuptools import setup, find_packages
import os

version = '2.0b3'
shortdesc = "Teaser/Banner content type for Plone"
longdesc = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()
longdesc += open(os.path.join(os.path.dirname(__file__), 'docs', 'HISTORY.rst')).read()


setup(name='collective.teaser',
      version=version,
      description=shortdesc,
      long_description=longdesc,
      # Get more strings from
      # http://pypi.python.org/pypi?:action=list_classifiers
      classifiers=[
        "Framework :: Plone",
        "Programming Language :: Python",
        ],
      keywords='Plone, Teaser, Banner',
      author='Johannes Raggam',
      author_email='raggam-nl@adm.at',
      url='http://github.com/collective/collective.teaser',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['collective'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          # -*- Extra requirements: -*-
          'archetypes.referencebrowserwidget',
          'plone.app.portlets',
          'node',
      ],
      entry_points="""
      # -*- Entry points: -*-
      [z3c.autoinclude.plugin]
      target = plone
      """,
      )
